package com.tzld.piaoquan.longarticle.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.vo.PublishMiniprogram;
import com.tzld.piaoquan.longarticle.service.CardService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Service
public class CardServiceImpl implements CardService {

    @ApolloJsonValue("${miniMap:{}}")
    private JSONObject miniProgramMap;

    @Autowired
    CoverServiceImpl coverService;


    public JSONObject createGzhPath(String videoId, String sharedUid, String ghId, Integer miniProgramType) {
        String rootShareId = UUID.randomUUID().toString();
        String rootSourceId;

        switch (miniProgramType) {
            case 2:
                rootSourceId = "touliu_tencentGzhArticle_" + ghId + "_" + generateSourceId();
                break;
            case 1:
                rootSourceId = "longArticles_" + generateSourceId();
                break;
            case 3:
                rootSourceId = "WeCom_" + generateSourceId();
                break;
            default:
                rootSourceId = "Error mini_program_type " + miniProgramType;
        }

        String url = String.format("pages/user-videos?id=%s&su=%s&fromGzh=1&rootShareId=%s&shareId=%s&rootSourceId=%s",
                videoId, sharedUid, rootShareId, rootShareId, rootSourceId);
        String productionPath = String.format("pages/category?jumpPage=%s", URLEncoder.encode(url, StandardCharsets.UTF_8));
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("rootShareId", rootShareId);
        jsonObject.put("rootSourceId", rootSourceId);
        jsonObject.put("productionPath", productionPath);
        return jsonObject;
    }

    private String generateSourceId() {
        long timestamp = System.currentTimeMillis();
        int randomNum = new Random().nextInt(9000) + 1000;
        String hashInput = timestamp + "-" + randomNum;
        return md5(hashInput);
    }

    private String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes(StandardCharsets.UTF_8));
            StringBuilder sb = new StringBuilder();
            for (byte b : messageDigest) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    public PublishMiniprogram generateSingleCard(Integer index, String ghId, Integer miniId, VideoDetail videoDetail, Integer miniProgramType, String publishContentId) {
        String strMiniId = String.valueOf(miniId);
        JSONObject miniInfo = miniProgramMap.getJSONObject(strMiniId);
        String avatar = miniInfo.getString("avatar");
        String appId = miniInfo.getString("id");
        String appName = miniInfo.getString("name");

        String videoId = videoDetail.getVideoId();
        String uid = videoDetail.getUid();
        JSONObject jsonObject = createGzhPath(videoId, uid, ghId, miniProgramType);

        PublishMiniprogram publishMiniprogram = new PublishMiniprogram();
        publishMiniprogram.setProgramId(appId);
        publishMiniprogram.setProgramName(appName);
        publishMiniprogram.setProgramAvatar(avatar);
        publishMiniprogram.setMini_id(miniId);
        publishMiniprogram.setMiniprogramUseType(miniProgramType);
        String kimiTitle = videoDetail.getKimiTitle();
        if (StringUtils.hasText(kimiTitle)) {
            kimiTitle = kimiTitle.replace("\"", "");
            if (kimiTitle.length() > 32) {
                kimiTitle = kimiTitle.substring(0, 32);
            }
            publishMiniprogram.setProductionName(kimiTitle);
        }
        String videoCover = videoDetail.getVideoCover();
        String imageUrl = coverService.pushCover(videoCover, publishContentId);
        if (StringUtils.isEmpty(imageUrl)) {
            //TODO 上线要放开
//            return null;
        }
        publishMiniprogram.setProductionCover(imageUrl);
        publishMiniprogram.setProductionPath(jsonObject.getString("productionPath"));
        publishMiniprogram.setRootShareId(jsonObject.getString("rootShareId"));
        publishMiniprogram.setSource(videoDetail.getSource());
        publishMiniprogram.setVideoUrl(videoDetail.getVideoPath());
        publishMiniprogram.setTraceId(videoDetail.getTraceId());
        publishMiniprogram.setParagraphPosition(BigDecimal.valueOf(index * 0.25));
        if (index == 1) {
            publishMiniprogram.setParagraphPosition(BigDecimal.valueOf(0.01));
        }
        return publishMiniprogram;
    }

    public List<PublishMiniprogram> generateCards(String ghId, List<VideoDetail> videoDetails, int miniProgramType, String publishContentId) {
        int long_articles_mini_program_id = 25;
        int touliu_mini_program_id = 33;
        int we_com_mini_program_id = 27;
        int miniId = 0;

        List<PublishMiniprogram> cardList = new ArrayList<>();

        switch (miniProgramType) {
            case 1:
                miniId = long_articles_mini_program_id;
                break;
            case 2:
                miniId = touliu_mini_program_id;
                break;
            case 3:
                miniId = we_com_mini_program_id;
                break;
            default:
                break;
        }
        if (miniId == 0) {
            return null;
        }
        for (int index = 0; index < videoDetails.size(); index++) {
            VideoDetail videoDetail = videoDetails.get(index);
            PublishMiniprogram card = generateSingleCard(index + 1, ghId, miniId, videoDetail, miniProgramType, publishContentId);
            if (card != null) {
                cardList.add(card);
            }
        }
        return cardList;
    }
}
