package com.tzld.piaoquan.longarticle.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.longarticle.model.bo.LongArticleSystemPlan;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.utils.HttpClientUtil;
import com.tzld.piaoquan.longarticle.utils.HttpPoolClientUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class PlanServiceImpl {

    private static final HttpPoolClientUtil HTTP_POOL_CLIENT_UTIL_DEFAULT = HttpClientUtil.create(5000, 5000, 20, 100, 3, 3000);


    public List<LongArticleSystemPlan> getAllLongArticleSystemPlan() {
        String apiUrl = "http://aigc-testapi.cybertogether.net/aigc/publish/LongArticleSystem/getAllLongArticleSystemPlan";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl);
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                JSONArray data = jsonObject.getJSONArray("data");
                List<LongArticleSystemPlan> list = data.toJavaList(LongArticleSystemPlan.class);
                return list;
            }
            return null;
        } catch (Exception e) {
            log.error("getAllLongArticleSystemPlan error", e);
        }
        return null;
    }

    public List<LongArticleSystemContentItemVO> getContentItemList(LongArticleSystemGetContentsParam param) {
        List<LongArticleSystemContentItemVO> contentItemList = new ArrayList<>();
        int pageNum = 1;
        int pageSize = 500;
        param.setPageNum(pageNum);
        param.setPageSize(pageSize);
        LongArticleSystemContentVO longArticleSystemContentVO = listWaitingMatchMiniprogramContents(param);
        if (longArticleSystemContentVO == null) {
            return new ArrayList<>();
        }
        if (!CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())) {
            contentItemList.addAll(longArticleSystemContentVO.getContentItemList());
        }
        Long totalCount = longArticleSystemContentVO.getTotalCount();
        while ((long) pageNum * pageSize < totalCount) {
            pageNum++;
            param.setPageNum(pageNum);
            LongArticleSystemContentVO longArticleSystemContentVO1 = listWaitingMatchMiniprogramContents(param);
            if (longArticleSystemContentVO1 != null && !CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())) {
                contentItemList.addAll(longArticleSystemContentVO.getContentItemList());
            }
        }
        return contentItemList;
    }

    public LongArticleSystemContentVO listWaitingMatchMiniprogramContents(LongArticleSystemGetContentsParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/publish/LongArticleSystem/listWaitingMatchMiniprogramContents";
        List<LongArticleSystemContentItemVO> contentItemList = new ArrayList<>();
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                LongArticleSystemContentVO longArticleSystemContentVO = jsonObject.getJSONObject("data").toJavaObject(LongArticleSystemContentVO.class);
                return longArticleSystemContentVO;
            }
        } catch (Exception e) {
            log.error("listWaitingMatchMiniprogramContents error", e);
        }
        return null;
    }

    public boolean createPushTask(LongArticleSystemCreatePushTaskParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/publish/LongArticleSystem/uploadMiniprogramCover";
        List<LongArticleSystemContentItemVO> contentItemList = new ArrayList<>();
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return true;
            }
        } catch (Exception e) {
            log.error("listWaitingMatchMiniprogramContents error", e);
        }
        return false;
    }

    public boolean filterSortContents(FilterSortParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/publish/LongArticleSystem/filterSortContents";
        List<LongArticleSystemContentItemVO> contentItemList = new ArrayList<>();
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return true;
            }
        } catch (Exception e) {
            log.error("listWaitingMatchMiniprogramContents error", e);
        }
        return false;
    }


}
